/* ------------------------------------------------------------ */
/* MqFileDown V001.001 --->                upd: 09.08.2011, und */
/* WebSphere MQ GET application                                 */
/* ------------------------------------------------------------ */
/* Requires: RXMQT.dll (Rexx-MQ-Routine-Client) <-------------- */
/*           RXMQN.dll (Rexx-MQ-Routine-Server)                 */
/* WMQ Support Pac MA95. Version 1.0.2                          */
/* (C) Copyright IBM Corporation. 1997, 2010                    */
/* ------------------------------------------------------------ */
/* - Read until end-of-queue                                    */
/* - Write message to file                                      */
/* ------------------------------------------------------------ */
parse arg inputParms
/* ------------------------------------------------------------ */
call FileSpecs
call InitFields
call InitFunctions
-- trace ?i
nop
call ConnectToMq         /* connect to MQ and open queue        */
if rcOpen = 0 then call ProcessQueue
call EndProcedure
exit
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
FileSpecs:
/* ------------------------------------------------------------ */
  OutPath    = "H:\_UK\MQ-Tools\Output\"
  LogPath    = "H:\_UK\MQ-Tools\Log\"
  ArchivPath = "H:\_UK\MQ-Tools\Archiv\"
  log_file   = "MQDownload.log"
  logFile = .stream~new(LogPath || log_file)
  do until logOpen = "READY:"
    logOpen = logFile~open
    if logOpen <> "READY:"
    then do
      say "===> RC-LogOpen =" logOpen
      say "===> ... waiting 5 sec..."
      Call SysSleep 5
    end
  end
return

/* ------------------------------------------------------------ */
InitFields:
/* ------------------------------------------------------------ */
  if inputParms~length = 0
  then do
    say "===================================================================>"
    say "===> Enter QueueManager, QueueName and Param for MQGET operation"
    say "===> Param-1: FileMode [0] first, [1] every"
    say "===> Param-2: Convert  [0] no,    [1] yes"
    say "===> Param-3: Number of Messages to read [0] all,"
    say "===> Example:"
    say "===> rexx MqFileDown.rex MQXNLF1 SYSLOG 0 0 0"
    say "===================================================================>"
    pull inputParms
  end
  if inputParms~length = 0 then exit -0815
  inputParms = inputParms~translate
  parse var inputParms qmgrName queueName mqParm
  if mqParm = "" then mqParm = "0 0 0"
  parse var mqParm fileModeEvery fileConvert numMsg
  if numMsg = "" then numMsg = 0
  if fileModeEvery then nop   /* exit here if value is not 0 or 1 */
  if fileConvert then nop

  call 'WMQClientConn.rex' qmgrName
  xFlag = result
  if xFlag = 99
  then do
    say "---> QueueManager       :" qmgrName "===> not found"
    exit -7777
  end
  if xFlag = 1 
  then msgLen = 4096
  else msgLen = 4194304

  OutFile   = qmgrName || "." || queueName || ".txt"
  OutStream = .stream~new(OutPath || OutFile)
  rcWrite = OutStream~open('write')
  say "===================================================================>"
  LogMsg.1 = "Application : RxMQGET , Target: QM =" qmgrName", Queue =" queueName
  LogMsg.2 = "-"~copies(80)
  LogMsg.0 = 1
  say "===>" LogMsg.1
  say "---> Start ====> :" time()
  call WriteToLog
  if queueName = "" then exit -9999
  msgCount = 0 ; byteCount = 0
return

/* ------------------------------------------------------------ */
InitFunctions:
/* ------------------------------------------------------------ */
  call rxfuncadd 'sysloadfuncs', 'rexxutil', 'sysloadfuncs'
  call sysloadfuncs
  rcc = RxFuncadd('RXMQINIT','RXMQT','RXMQINIT')
  parse var rcc rcFunc .
  if rcFunc <> 0 & rcFunc <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Load MQ Functions"
    call WriteToLog
    exit -3333
  end
  rcc = RXMQINIT()
  rcc = RXMQCONS()
  parse var rcc rcInit .
  if rcInit <> 0 & rcInit <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Init MQ Functions"
    call WriteToLog
    exit -4444
  end
Return

/* --------------------------------------------------------- */
ConnectToMq:
/* --------------------------------------------------------- */
  rcc = RXMQCONN(qmgrName)
  parse var rcc rcConn .
  if rcConn = 0
  then do
    rcc = RXMQOPEN(queueName, mqoo_input_shared, 'ObjHndl', 'ood.')
    parse var rcc rcOpen .
    if rcOpen <> 0
    then do
      LogMsg.0 = 2
      LogMsg.1 = "MQOPEN failed with" rcc
      call WriteToLog
      exit -4444
    end
  end
  else do
    LogMsg.0 = 2
    LogMsg.1 = "MQCONN failed with" rcc
    call WriteToLog
    exit -4444
  end
  say "---> Open Queue  :" rcOpen
return ConnectToMq

/* ------------------------------------------------------------ */
ProcessQueue:
/* ------------------------------------------------------------ */
  if xFlag = 1
  then igmo.opt = MQGMO_WAIT + MQGMO_ACCEPT_TRUNCATED_MSG + MQGMO_SYNCPOINT
  else igmo.opt = MQGMO_WAIT + MQGMO_ACCEPT_TRUNCATED_MSG + MQGMO_SYNCPOINT + MQGMO_CONVERT
       
  rcGetOK = 1
  do until rcGetOK <> 1
    g.0 = msgLen
    g.1 = ''
    rcc = RXMQGET( ObjHndl,'g.','igmd.','ogmd.','igmo.','ogmo.')
    parse var rcc rcGet rcWarn .
    rcOp = rcGet
    if rcGet = 0 | rcWarn = 1
    then do
      if rcGet = 2079
      then do
        LogMsg.0 = 1
        LogMsg.1 = "Message truncated" rcc
        call WriteToLog
      end
      if rcGet = 2110
      then do
        LogMsg.0 = 1
        LogMsg.1 = "Message Format Error" rcc
        call WriteToLog
      end
      call MessageToFile
      msgCount = msgCount + 1
      byteCount = byteCount + g.0
      if numMsg > 0 & numMsg = msgCount
      then do
        rcGet = 2033
        say "---> Get Message : 0 [End_of_Queue by MessageCounter]"
        rcGetOK = 0
      end
    end
    else do
      if rcGet = 2033
      then say "---> Get Message : 0 [End_of_Queue]"
      else say "---> Get Failed  :" rcGet
      rcGetOK = 0
    end
  end

  LogMsg.0 = 1
  LogMsg.1 = "MQGET ["OutFile"] completed with rc" rcGet". Msgs:" msgCount ", Bytes:" byteCount
  call WriteToLog
  say "---> Processed   :" msgCount
  qdta.0 = msgCount
Return

MessageToFile:
/* --------------------------------------------------------- */
/* Write to file                                             */
/* --------------------------------------------------------- */
  if fileConvert
  then do
    call 'RxTrans' g.1, e, a
    g.1 = result
  end

  if fileModeEvery
  then do
    OutFile   = qmgrName || "." || queueName || ".txt"
    OutStream = .stream~new(OutPath || OutFile)
    rcWrite = OutStream~open('write')
  end
  rcWrite = OutStream~lineout(g.1)
  if rcWrite = 0
  then do
    rcc = RXMQCMIT()
    parse var rcc rcCmt .
    if rcCmt <> 0
    then do
      say "---> Commit Err  :" rcc
      LogMsg.0 = 1
      LogMsg.1 = "Commit failed with" rcc
      call WriteToLog
    end
  end
  else do
    rcc = RXMQBACK()
    parse var rcc rcBck .
    if rcBck <> 0
    then do
      say "---> Backout Err :" rcc
      LogMsg.0 = 1
      LogMsg.1 = "Backout failed with" rcc
      call WriteToLog
    end
    LogMsg.0 = 2
    LogMsg.1 = "Backout required due to io error" rcWrite
    call WriteToLog
    exit -9898
  end
  if fileModeEvery then OutStream~close

  /* mqHeader = queueName "[" || msgCount~right(9) || "]" "PutDate :" ogmd.pd ", Length :" g.0 */
  /* ---> without dead-letter-header OutStream~lineout(mqData~substr(173)) */
  /* qdta.msgCount = mqHeader || "0D0A"x || mqData */
  /* qdta.msgCount = g.1~substr(173)               */
  /* OutStream~charout(g.1~substr(173))            */
  /* OutStream~lineout(qdta.msgCount)              */
  /* OutStream~lineout('')                         */
return MessageToFile

/* --------------------------------------------------------- */
/* Write to logfile                                          */
WriteToLog:
/* --------------------------------------------------------- */
  TimeStamp = "["date()~right(11) time()"] --->"
  if logOpen <> "READY:" then exit -7777
  if LogMsg.0 > 0
  then do LogCount = 1 to LogMsg.0
    LogFile~lineout(TimeStamp LogMsg.LogCount)
  end
return WriteToLog

/* ------------------------------------------------------------ */
EndProcedure:
/* ------------------------------------------------------------ */
  LogMsg.0 = 1
  LogMsg.1 = LogMsg.2
  call WriteToLog
  LogFile~close
  OutStream~close
  if rcOpen = 0
  then do
    rcc = RXMQCLOS(ObjHndl, mqco_none)
    parse var rcc rcClose .
    say "---> Close Queue :" rcClose
  end
  if rcConn = 0
  then do
    rcc = RXMQDISC()
    parse var rcc rcDisc .
    say "---> Disconnect  : Conn("rcConn") Open("rcOpen") Oper("rcOp") Disc("rcDisc")"
    say "---> File        :" OutFile
    say "---> End ======> :" time()
    say "===================================================================>"
  end
Return
