/* ------------------------------------------------------------ */
/* MqLineUp V001.001 --->                       09.03.2012, und */
/* WebSphere MQ PUT application                                 */
/* ------------------------------------------------------------ */
/* Requires: RXMQT.dll (Rexx-MQ-Routine-Client)                 */
/*           RXMQN.dll (Rexx-MQ-Routine-Server)                 */
/*           RxTrans.rex (EBCDIC to ASCII conversion)           */
/* WMQ Support Pac MA95. Version 1.02 (C)                       */
/* Copyright IBM Corporation. 1997, 2008, 2010.                 */
/* ------------------------------------------------------------ */
/* ------------------------------------------------------------ */
/* - Processes files in input directory                         */
/* - Generate one message per line (end with CRLF               */
/* ------------------------------------------------------------ */
parse arg parms

/* ------------------------------------------------------------ */
call FileSpecs        /* input and output file specification    */
call InitFields       /* init fields                            */
call InitFunctions    /* init MQ                                */
call ConnectToMq      /* connect to MQ and open queue           */
if rcConn = 0 & rcOpen = 0
then call ProcessFiles/* process .txt files in directory        */
call EndProcedure     /* close files an terminate MQ            */
exit                  /* End                                    */

/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
FileSpecs:
/* ------------------------------------------------------------ */
  file_spec   = "*.*"
  inp_path    = "H:\_UK\MQ-Tools\Input\"
  log_path    = "H:\_UK\MQ-Tools\Log\"
  archiv_path = "H:\_UK\MQ-Tools\Archiv\"
  log_file    = "MQUpload.log"
  logFile = .stream~new(log_path || log_file)
  do until log_open = "READY:"
    log_open = logFile~open
    if log_open <> "READY:"
    then do
      say "RC-LogOpen =" log_open
      say "... waiting 5 sec..."
      Call SysSleep 5
    end
  end
  call sysfiletree inp_path || file_spec, 'file', FO
  if file.0 = 0
  then fileMsg = "no Files found for MQ-Upload: <Terminate>"
  else fileMsg = "Files found for MQ-Upload:" file.0
return

/* ------------------------------------------------------------ */
InitFields:
/* ------------------------------------------------------------ */
  if parms~length = 0
  then do
    say "===================================================================>"
    say "===> Enter QueueManager and QueueName for MQPUT"
    say "===> Example:"
    say "===> rexx MqLineUp.rex MQXNLF1 SYSLOG"
    say "===================================================================>"
    say "===>" fileMsg
    say "===================================================================>"
    pull parms
  end
  if parms~length = 0 then exit -0815
  if file.0 = 0 then exit -4711
  parms = parms~translate
  parse var parms qmgrName queueName .
  call 'WMQClientConn.rex' qmgrName
  xFlag = result
  if xFlag = 99
  then do
    say "---> QueueManager       :" qmgrName "===> not found"
    exit -7777
  end
  say "===================================================================>"
  say "===> QueueManager is" qmgrName "xFlag is" xFlag
  LogMsg.1 = "Application: RxMQPUT , Target: QM =" qmgrName", Queue =" queueName
  LogMsg.2 = "-"~copies(80)
  LogMsg.0 = 1
  say "===>" LogMsg.1
  call WriteToLog
  fileCount = 0 ; byteCount = 0
  /* Commit after ... msgs */
  cmitLevel = 10000
return

/* ------------------------------------------------------------ */
InitFunctions:
/* ------------------------------------------------------------ */
  call rxfuncadd 'sysloadfuncs', 'rexxutil', 'sysloadfuncs'
  call sysloadfuncs
  rcc = RxFuncadd('RXMQINIT','RXMQT','RXMQINIT')
  parse var rcc rcFunc .
  if rcFunc <> 0 & rcFunc <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Load MQ Functions"
    call WriteToLog
    exit -3333
  end
  rcc = RXMQINIT()
  rcc = RXMQCONS()
  parse var rcc rcInit .
  if rcInit <> 0 & rcInit <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Init MQ Functions"
    call WriteToLog
    exit -4444
  end
return

/* ------------------------------------------------------------ */
ConnectToMq:
/* ------------------------------------------------------------ */
  rcc = RXMQCONN(qmgrName)
  parse var rcc rcConn .
  if rcConn = 0
  then do
    iod.ot = MQOT_Q
    rcc = RXMQOPEN(queueName, mqoo_output, 'ObjHndl', 'ood.')
    parse var rcc rcOpen .
    if rcOpen <> 0
    then do
      LogMsg.0 = 2
      LogMsg.1 = "MQOPEN failed with" rcc
      call WriteToLog
      exit -4444
    end
  end
  else do
    LogMsg.0 = 2
    LogMsg.1 = "MQCONN failed with" rcc
    call WriteToLog
    exit -4444
  end
return

/* ------------------------------------------------------------ */
ProcessFiles:
/* ------------------------------------------------------------ */
  do i = 1 to file.0
    lineCount = 0 ; byteCount = 0
    in_file      = file.i
    in_file_name = in_file~substr(in_file~lastpos("\") + 1)
    in_file_path = in_file~substr(1,in_file~lastpos("\"))
    if in_file_name~lastpos(".") > 0
    then in_file_noex = in_file_name~substr(1,in_file_name~lastpos(".") - 1)
    else in_file_noex = in_file_name
    in_file_err  = in_file_path || in_file_noex || ".err"
    err_file     = in_file_path || in_file_err
    mq_inp = .stream~new(in_file)
    rc = mq_inp~open('read')
    if rc <> 'READY:'
    then do
      LogMsg.0 = 1
      LogMsg.1 = "Error open Input file" in_file_name
      call WriteToLog
      iterate
    end
    do while mq_inp~lines > 0
      g.1 = mq_inp~linein
      g.0 = g.1~length
      if xFlag = 1
      then do
        call 'RxTrans' g.1, a, e
        g.1 = result
      end
      -- MQRO_EXPIRATION = 300
      -- ipmd.exp = MQRO_EXPIRATION
      ipmd.MSG = MQMT_DATAGRAM
      ipmd.form = 'MQSTR'
      ipmd.PER = MQPER_PERSISTENT
      ipmo.opt = MQPMO_SYNCPOINT + MQPMO_NO_CONTEXT + MQPMO_FAIL_IF_QUIESCING
      rcc = RXMQPUT( ObjHndl,'g.','ipmd.','opmd.','ipmo.','opmo.')
      parse var rcc rcPut .
      if rcPut = 0
      then do
        lineCount = lineCount + 1
        byteCount = byteCount + g.0
        /* commit messages after ... */
        if lineCount // cmitLevel = 0
        then do
          if cmitLevel > 2000 then say "===>" time() lineCount "===> Commit now"
          rcc = RXMQCMIT()
        end
      end
      else do
        LogMsg.0 = 2
        LogMsg.1 = "MQPUT ["in_file_name"] failed with" rcc "[Line:" || lineCount || ":"byteCount":]"
        call WriteToLog
        exit -4444
      end
    end
    /* Commit after last message */
    /* rcc = RXMQCMIT() */
    LogMsg.0 = 1
    LogMsg.1 = "MQPUT ["in_file_name"] completed with rc" rcPut". Lines:" lineCount "Bytes:" byteCount
    call WriteToLog
    say LogMsg.1
    mq_inp~close
    if rcPut = 0
    then rcMov = SysFileMove(in_file, archiv_path || in_file_name) /* "MOVE" in_file archiv_path */
    else do
      rcRen = SysFileCopy(in_file, in_file_err)
      rcRen = SysFileDelete(in_file)
    end
    if rcMov <> 0
    then do
      LogMsg.0 = 2
      LogMsg.1 = "MOVE failed with RC" rcMov "["in_file_name"]"
      call WriteToLog
      exit -5555
    end
  end
return

/* --------------------------------------------------------- */
/* Write to logfile                                          */
WriteToLog:
/* --------------------------------------------------------- */
  TimeStamp = "["date()~right(11) time()"] --->"
  if log_open <> "READY:" then exit -7777
  if LogMsg.0 > 0
  then do LogCount = 1 to LogMsg.0
    LogFile~lineout(TimeStamp LogMsg.LogCount)
  end
return WriteToLog

/* ------------------------------------------------------------ */
EndProcedure:                 /* close files an terminate MQ    */
/* ------------------------------------------------------------ */
  LogMsg.0 = 1
  LogMsg.1 = LogMsg.2
  call WriteToLog
  LogFile~close
  rcc = RXMQCLOS(ObjHndl, mqco_none)
  rcc = RXMQDISC()
  say "===================================================================>"
return
