/* ------------------------------------------------------------ */
/* MqFilePut V001.001 --->                 upd: 09.07.2012, und */
/* WebSphere MQ PUT application                                 */
/* Test : MQPMO_PASS_ALL_CONTEXT MQOO_SET_ALL_CONTEXT           */
/*        ipmd.exp = 1000 (Expiry)                              */
/* ------------------------------------------------------------ */
/* Requires: RXMQT.dll (Rexx-MQ-Routine-Client) <-------------- */
/*           RXMQN.dll (Rexx-MQ-Routine-Server)                 */
/* WMQ Support Pac MA95. Version 1.02 (C)                       */
/* Copyright IBM Corporation. 1997, 2008, 2010.                 */
/* ------------------------------------------------------------ */
/* - Processes files in input directory                         */
/* - Generate one message per file                              */
/* ------------------------------------------------------------ */
parse arg parms

/* ------------------------------------------------------------ */
call FileSpecs        /* input and output file specification    */
call InitFields       /* init fields                            */
call InitFunctions    /* init MQ                                */
call ConnectToMq      /* connect to MQ and open queue           */
if rcConn = 0 & rcOpen = 0
then call ProcessFiles/* process .txt files in directory        */
call EndProcedure     /* close files an terminate MQ            */
say "="~copies(80)">"
say "===> End-Of-Procedure"
exit                  /* End                                    */
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
FileSpecs:
/* ------------------------------------------------------------ */
  file_spec   = "*.*"
  inp_path    = "H:\_UK\MQ-Tools\Input\"
  log_path    = "H:\_UK\MQ-Tools\Log\"
  archiv_path = "H:\_UK\MQ-Tools\Archiv\"
  log_file    = "MQUpload.log"
  logFile = .stream~new(log_path || log_file)
  do until log_open = "READY:"
    log_open = logFile~open
    if log_open <> "READY:"
    then do
      say "===> RC-LogOpen =" log_open
      say "===> ... waiting 5 sec..."
      Call SysSleep 5
    end
  end
  call sysfiletree inp_path || file_spec, 'file', FO
  if file.0 = 0
  then do
    say "===> no Files found for MQ-Upload:"
    pull .
    /* exit -9999 - leave if directory is empty */
  end
  say "===> Files found for MQ-Upload:" file.0
return

/* ------------------------------------------------------------ */
InitFields:
/* ------------------------------------------------------------ */
  if parms~length = 0
  then do
    say "="~copies(80)">"
    say "===> Enter QmgrName, QueueName and Param for MQPUT operation"
    say "===> Param-1: FileMode [0] first, [1] every"
    say "===> Param-2: Convert  [0] no,    [1] yes"
    say "===> Example:"
    say "===> rexx MqFileUp.rex MQXNLF1 SYSLOG 0 0"
    say "="~copies(80)">"
    pull parms
  end
  if parms~length = 0 then exit -0815
  parms = parms~translate
  parse var parms qmgrName queueName mqParm
  if mqParm = "" then mqParm = "0 0"
  parse var mqParm fileModeEvery fileConvert
  if fileModeEvery then nop   /* exit here if value is not 0 or 1 */
  if fileConvert then nop
  call 'WMQClientConn.rex' qmgrName
  zSystem = result
  if zSystem = 99
  then do
    say "---> QueueManager       :" qmgrName "===> not found"
    exit -7777
  end
  say "="~copies(80)">"
  LogMsg.1 = "Application: RxMQPUT , Target: QM =" qmgrName", Queue =" queueName
  LogMsg.2 = "-"~copies(80)
  LogMsg.0 = 1
  say "===>" LogMsg.1
  call WriteToLog
  if queueName = "" then exit -9999
return InitFields

/* ------------------------------------------------------------ */
InitFunctions:
/* ------------------------------------------------------------ */
  call rxfuncadd 'sysloadfuncs', 'rexxutil', 'sysloadfuncs'
  call sysloadfuncs
  rcc = RxFuncadd('RXMQINIT','RXMQT','RXMQINIT')
  parse var rcc rcFunc .
  if rcFunc <> 0 & rcFunc <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Load MQ Functions"
    call WriteToLog
    exit -3333
  end
  rcc = RXMQINIT()
  rcc = RXMQCONS()
  parse var rcc rcInit .
  if rcInit <> 0 & rcInit <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Init MQ Functions"
    call WriteToLog
    exit -4444
  end
return

/* ------------------------------------------------------------ */
ConnectToMq:
/* ------------------------------------------------------------ */
  rcc = RXMQCONN(qmgrName)
  parse var rcc rcConn .
  if rcConn = 0
  then do
    /* MQPMO_PASS_ALL_CONTEXT MQOO_SET_ALL_CONTEXT */
    ood.dqn  = "HUGO"
    -- ood.rqmn = "MQT1"
    -- ood.ot   = MQOT_Q
    oOpt = mqoo_output + mqpmo_pass_all_context + mqoo_set_all_context
    rcc = RXMQOPEN(queueName, oOpt, 'ObjHndl', 'ood.')
    parse var rcc rcOpen .
    if rcOpen <> 0
    then do
      LogMsg.0 = 2
      LogMsg.1 = "MQOPEN failed with" rcc
      call WriteToLog
      exit -4444
    end
  end
  else do
    LogMsg.0 = 2
    LogMsg.1 = "MQCONN failed with" rcc
    call WriteToLog
    exit -4444
  end
return

/* ------------------------------------------------------------ */
ProcessFiles:
/* ------------------------------------------------------------ */
  do i = 1 to file.0
    in_file      = file.i
    in_file_name = in_file~substr(in_file~lastpos("\") + 1)
    in_file_path = in_file~substr(1,in_file~lastpos("\"))
    if in_file_name~lastpos(".") > 0
    then in_file_noex = in_file_name~substr(1,in_file_name~lastpos(".") - 1)
    else in_file_noex = in_file_name
    in_file_err  = in_file_path || in_file_noex || ".err"
    err_file     = in_file_path || in_file_err
    mq_inp = .stream~new(in_file)
    rc = mq_inp~open('read')
    if rc <> 'READY:'
    then do
      LogMsg.0 = 1
      LogMsg.1 = "Error open Input file" in_file_name
      call WriteToLog
      exit -4444
    end
    rcMov = 0 ; rcDel = 0 ; rcRen = 0
    BufferLength = mq_inp~chars
    MsgBuffer = mq_inp~charin(,BufferLength)
    g.0 = BufferLength
    g.1 = MsgBuffer
    if fileConvert
    then do
      call 'RxTrans' g.1, a, e
      g.1 = result
    end
    
    ipmd.form = 'MQSTR'
    ipmd.per  = MQPER_PERSISTENT
    ipmo.opt  = MQPMO_SYNCPOINT
    ipmd.exp  = 10000
    rcc = RXMQPUT( ObjHndl,'g.','ipmd.','opmd.','ipmo.','opmo.')
    parse var rcc rcPut .
    /* ====> Achtung ====> */
    if rcPut = 0
    then do
      if i // 100 = 0
      then do
        say "===>" i "Commit now"
        rcc = RXMQCMIT()
      end
      LogMsg.0 = 1
      LogMsg.1 = "MQPUT ["in_file_name"] completed with rc" rcPut". BytesTotal:" BufferLength
      call WriteToLog
      say "MQPUT ["rcPut"]  Bytes ["g.0~right(9)"] ["in_file_name"]"
    end
    else do
      LogMsg.0 = 2
      LogMsg.1 = "MQPUT failed with" rcc "["in_file_name"]"
      call WriteToLog
      exit -5512
    end
    mq_inp~close
    if rcPut = 0
    then do
      rcDel = SysFileDelete(archiv_path || in_file_name)
      if rcDel = 0 then say "===> Duplicate File in Arcive-Directory deleted:"
      rcMov = SysFileMove(in_file, archiv_path || in_file_name)
    end
    else do
      rcRen = SysFileCopy(in_file, in_file_err)
      rcRen = SysFileDelete(in_file)
    end
    if rcMov <> 0
    then do
      LogMsg.0 = 2
      LogMsg.1 = "MOVE failed with RC" rcMov
      call WriteToLog
      exit -5555
    end
  end
return

/* ------------------------------------------------------------ */
/* Write to logfile                                             */
WriteToLog:
/* ------------------------------------------------------------ */
  TimeStamp = "["date()~right(11) time()"] --->"
  if log_open <> "READY:" then exit -7777
  if LogMsg.0 > 0
  then do LogCount = 1 to LogMsg.0
    LogFile~lineout(TimeStamp LogMsg.LogCount)
  end
return WriteToLog

/* ------------------------------------------------------------ */
EndProcedure:                 /* close files an terminate MQ    */
/* ------------------------------------------------------------ */
  LogMsg.0 = 1
  LogMsg.1 = LogMsg.2
  call WriteToLog
  LogFile~close
  rcc = RXMQCLOS(ObjHndl, MQCO_NONE)
  rcc = RXMQDISC()
return

/* ------------------------------------------------------------ */
/* DumpVars:                                                    */
/* ------------------------------------------------------------ */
/*  var_File = "H:_UK\MQ-Tools\log\MyVars.txt"                  */
/*  Call SysDumpVariables var_File                              */
/* return                                                       */
/* ------------------------------------------------------------ */
/*    qmgrName = "MQANL51"                                     */
/*    "SET MQCCSID=850"                                         */
/*    "SET MQSERVER=MQANL51.CH/TCP/anl5.vme.vse.dresdner.net"   */
/* ------------------------------------------------------------ */
