/* ------------------------------------------------------------ */
/* Rexx Webshere MQ Browse-Get-Copy-Move utility 10.03.09, und  */
/* Using client connection channels to VSE and non VSE QM       */
/* ------------------------------------------------------------ */
/* [B]rowse, [G]et from queue to file                           */
/* [C]opy, [M]ove from one queue to another queue, removing     */
/* DEAD-LETTER-HEADER and XMT-HEADER                            */
/* Usage Examples:                                              */
/* Browse B MQFFM01 TEST.QUEUE ---> messages to file            */
/*                                  MQFFM01.TEST.QUEUE          */
/* Get    G MQFFM01 TEST.QUEUE ---> messages to file            */
/*                                  MQFFM01.TEST.QUEUE          */
/* Copy   C MQFFM01 TEST.QUEUE CLE.DIRECTDEB                    */
/*                                  Copy from queue to queue    */
/* Move   M MQFFM01 TEST.QUEUE CLE.DIRECTDEB                    */
/*                                  Move from queue to queue    */
/* Copy   C MQFFM01 MQFFM01.DEAD.LETTER.QUEUE *                 */
/* or Move                          Restore to original queue   */
/* ------------------------------------------------------------ */
/*                             Option for GET, BROWSE           */
/* Browse B MQFFM01 TEST.QUEUE 0  ---> Output with header       */
/* Browse B MQFFM01 TEST.QUEUE 1  ---> noheader in one file     */
/* Browse B MQFFM01 TEST.QUEUE 2  ---> noheader in each file    */
/* ------------------------------------------------------------ */
/* Requires: RXMQT.dll (Rexx-MQ-Routine-Client)                 */
/*           RxTrans.rex (EBCDIC to ASCII conversion)           */
/* WMQ Support Pac MA95. Version 1.02 (C)                       */
/* Copyright IBM Corporation. 1997, 2008, 2010.                 */
/* ------------------------------------------------------------ */
parse arg parms
-- trace ?i
/* ------------------------------------------------------------ */
call PrgInit
call SetWmqServer
call FileInit
call WmqInit
call ProcessQueue
call EndProcedure
exit
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
PrgInit:
/* ------------------------------------------------------------ */
  rcInit = 999 ; rcConn = 999 ; rcOpen = 999 ; rcGet = 999 ; rcPut = 999 ; rcClose = 999 ; rcDisc = 999 ; lastOper = 999
  rcPut = "-99" ; rcOpu = "-99"
  hLine = "="~copies(76) || ">"
  if parms~length = 0
  then do
    say ""
    say hLine
    say "MqUtil.rex  ===> WebSphereMQ utility program"
    say "Params      ===> BROWSE GET COPY MOVE [B,G,C,M]"
    say "            ===> QueueManager"
    say "            ===> GetQueue"
    say "            ===> PutQueue"
    say "----------- ===>" "-"~copies(60)
    say "BROWSE and  ===> 0 [default] output with header"
    say "GET options ===> 1 noheader, 2 noheader msg per file"
    say "----------- ===>" "-"~copies(60)
    say "Usage       ===> B  MQFFM01 TEST.QUEUE [0] ---> one file with header"
    say "            ===> BN MQFFM01 TEST.QUEUE [0] ---> one file with header -noconvert-"
    say "            ===> B  MQFFM01 TEST.QUEUE 1   ---> one file no header"
    say "            ===> B  MQFFM01 TEST.QUEUE 2   ---> multiple files no header"
    say "            ===> M  MQFFM01 TEST.01 TEST.02"
    say "            ===> B  MQANLC1 ANYQ"
    say "DLQ-Restore ===> M  MQFFM01 MQFFM01.DEAD.LETTER.QUEUE *"
    say hLine
    exit -2222
  end
  else do
    parms = parms~translate
    parse var parms mqOpt qmgrName GetQueue PutQueue
    parse var parms mqOpt qmgrName GetQueue outFileOpt
  end
  j = 0
return

/* ------------------------------------------------------------ */
SetWmqServer:
/* ------------------------------------------------------------ */
  /* ---> Non VSE Server -------------------------------------- */
  msgLen = 104857600
  msgLen = 4194304
  call 'WMQClientConn.rex' qmgrName
  parse var result xFlag qmgrName
  if xFlag = 99
  then do
    say "---> QueueManager       :" qmgrName "===> not found"
    exit -7777
  end
  if xFlag > 10 then msgLen = 4096
return

/* ------------------------------------------------------------ */
FileInit:
/* ------------------------------------------------------------ */
  if mqOpt = "BN"
    /* Browse with noconvert */
  then do
    mqOpt = "B"
    xFlag = 1
  end
  if mqOpt = "B" | mqOpt = "G"
  then do
    OutPath = "H:\_UK\MQ-Tools\Output\"
    OutFile = qmgrName || "." || GetQueue
    OutFileExt = ".txt"
    select
      when outFileOpt = 1
      then do
        OutStream = .stream~new(OutPath || OutFile || OutFileExt)
        rcWrite = OutStream~open('write')
      end
      when outFileOpt = 2
      then do
      end
      otherwise do
        outFileOpt = 0
        OutStream = .stream~new(OutPath || OutFile || OutFileExt)
        rcWrite = OutStream~open('write')
      end
    end
  end
  say ""
  say hLine
  say "---> QueueManager       :" qmgrName
  say "---> GetQueueName       :" GetQueue
return

/* ------------------------------------------------------------ */
WmqInit:
/* ------------------------------------------------------------ */
  rcc = rxfuncadd('RXMQINIT','RXMQT','RXMQINIT')
  rcc = RXMQINIT()
  parse var rcc rcInit .
  rcc = RXMQCONN(qmgrName)
  parse var rcc rcConn .
  rcc = RXMQCONS()
  if rcConn <> 0
  then do
    say "---> ConnectFailed      :" rcConn
    exit -9999
  end
Return

/* ------------------------------------------------------------ */
ProcessQueue:
/* ------------------------------------------------------------ */
  /* Open queue for BROWSE */
  rcc = RXMQOPEN(GetQueue, mqoo_input_shared + mqoo_browse, 'h2', 'ood.')
  parse var rcc rcOpen .
  say "---> OpenForGet         :" rcOpen
  rcGet = 0 ; rcSplit = 1 ; rcCheck = 0
  if rcOpen = 0
  then do until rcCheck <> 0
    gMsg.0 = msgLen ; gMsg.1 = ''
    /* bei Format Error 2110 ohne CONVERT benutzen */
    if xFlag = 1
    then igmo.opt = MQGMO_WAIT + MQGMO_BROWSE_NEXT + MQGMO_ACCEPT_TRUNCATED_MSG
    else igmo.opt = MQGMO_WAIT + MQGMO_BROWSE_NEXT + MQGMO_ACCEPT_TRUNCATED_MSG + MQGMO_CONVERT
    /* first Get with MQGMO_BROWSE_NEXT */
    lastOper = "GET"
    rcc = RXMQGET( h2,'gMsg.','igmd.','ogmd.','igmo.','ogmo.')
    parse var rcc rcGet .
    if rcGet = 2079
    then do
      say  "---> Message truncated :" rcGet
      rcGet = 0000
    end
    rcCheck = rcGet
    if rcGet = 0
    then do
      -- say  ":===>:" OGMD.EXP
      j = j + 1
      pMsg.0 = gMsg.0
      pMsg.1 = gMsg.1
      if rcSplit = 0 then call CheckMsgHeader
      if mqOpt = "B" | mqOpt = "G"
      then do
        /* ---> Message to file */
        if xFlag = 11
          /* ---> Translate EBCDIC to ASCII */
        then do
          call 'RxTrans' gMsg.1, e, a
          gMsg.1 = result
        end
        call MessageToFile
        if rcWrite = 0 & mqOpt = "G"
        then call DeleteMessage
      end
      if mqOpt = "C" | mqOpt = "M"
      then do
        call PutQueue
        if rcCheck = 0 & mqOpt = "M"
        then call DeleteMessage
      end
      rcc = RXMQCMIT()
      parse var rcc rcCmit .
      if rcCmit <> 0 then say "---> CommitMessage      :" rcCmit
    end
  end
  if rcGet = 2033
  then say "---> GetReturn          : 0 [End_of_Queue]"
  else say "---> GetFailed          :" rcGet
  if rcOpen = 0
  then do
    rcc = RXMQCLOS(h2, mqco_none)
    parse var rcc rcClose .
    say "---> CloseQueue         :" rcClose
  end
  say "---> Processed          :" j
  qdta.0 = j
Return

/* ------------------------------------------------------------ */
MessageToFile:
/* ------------------------------------------------------------ */
  mqData = gMsg.1
  if outFileOpt = 0
  then do
    putDT = ogmd.pd~substr(1,4) || '-' || ogmd.pd~substr(5,2) || '-' || ogmd.pd~substr(7,2)
    putTM = ogmd.pt~substr(1,2) || ':' || ogmd.pt~substr(3,2) || ':' || ogmd.pt~substr(5,2)
    mqHeader = '[' || j~right(4) || '] PutDate :' putDT ', PutTime :' putTM ', Length :' gMsg.0 ', Data <'
    qdta.j = mqHeader mqData
    rcWrite = OutStream~charout(mqHeader || mqData  || '>')
    rcWrite = OutStream~lineout('')
    rcCheck = rcWrite
  end
  if outFileOpt = 1
  then do
    rcWrite = OutStream~charout(mqData)
    rcWrite = OutStream~lineout('')
    rcCheck = rcWrite
  end
  if outFileOpt = 2
  then do
    countFile = 100000 + j
    OutStream = .stream~new(OutPath || OutFile || "." || countFile || OutFileExt)
    rcWrite = OutStream~open('write')
    if rcWrite = "READY:"
    then do
      OutStream~charout(mqData)
      OutStream~close
    end
    else do
      say "---> File Open Failed   :" OutStream
      say "---> File Name          :" countFile || OutFileExt
    end
  end
Return

/* ------------------------------------------------------------ */
CheckMsgHeader:
/* ------------------------------------------------------------ */
  msgType = ""
  rcc = RXMQHXT( "gMsg.", "splitUP.")
  parse var rcc rcSplit .
  if rcSplit = 0
  then do
    pMsg.0 = splitUP.0
    pMsg.1 = splitUP.1
    if splitUP.TYPE = "XQH"
    then do
      remoteQM = splitUP.RQM
      remoteQN = splitUP.RQN
    end
    if splitUP.TYPE = "DLH"
    then do
      dlhReason = splitUP.REA
      dlhDestQM = splitUP.DQM
      dlhDestQN = splitUP.DQN
      dlhEncodi = splitUP.ENC
      dlhCodCar = splitUP.CCSI
      dlhFormat = splitUP.FORM
      dlhPutApT = splitUP.PAT
      dlhPutApN = splitUP.PAN
      dlhPutDat = splitUP.PD
      dlhPutTim = splitUP.PT
      if putQueue = "*" then putQueue = dlhDestQN
    end
  end
Return
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
PutQueue:
/* ------------------------------------------------------------ */
  rcc = RXMQOPEN(PutQueue, mqoo_output, 'h3', 'ood.')
  parse var rcc rcOpu .
  ipmd.PER = MQPER_PERSISTENT
  ipmd.form = 'MQSTR'
  ipmo.opt = MQPMO_SYNCPOINT
  lastOper = "PUT"
  rcc = RXMQPUT( h3,'pMsg.','ipmd.','opmd.','ipmo.','opmo.')
  parse var rcc rcPut .
  if rcPut <> 0 then j = j - 1
  rcc = RXMQCLOS(h3, mqco_none)
  parse var rcc rcClo .
  rcCheck = rcOpu + rcPut + rcClo
Return

/* ------------------------------------------------------------ */
DeleteMessage:
/* ------------------------------------------------------------ */
  /* ---> Delete the message */
  dMsg.0 = gMsg.0
  if xFlag = 1
  then igmo.opt = MQGMO_WAIT + MQGMO_ACCEPT_TRUNCATED_MSG + MQGMO_MSG_UNDER_CURSOR
  else igmo.opt = MQGMO_WAIT + MQGMO_ACCEPT_TRUNCATED_MSG + MQGMO_MSG_UNDER_CURSOR + MQGMO_CONVERT
  rcc = RXMQGET( h2,'dMsg.','igmd.','ogmd.','igmo.','ogmo.')
  parse var rcc rcDel .
  if rcDel <> 0 then j = j - 1
  -- say "---> MsgUnderCursor     :" rcDel
  rcCheck = rcDel
Return

/* ------------------------------------------------------------ */
EndProcedure:
/* ------------------------------------------------------------ */
  if mqOpt = "G" | mqOpt = "B" then OutStream~close
  if mqOpt = "C" | mqOpt = "M"
  then do
    say "---> PutQueueName       :" PutQueue
    say "---> OpenForPut         :" rcOpu
    say "---> PutReturn          :" rcPut
  end
  if rcConn = 0
  then do
    rcc = RXMQDISC()
    parse var rcc rcDisc .
    say "---> Disconnect         : Conn("rcConn") Open("rcOpen") LastOper("lastOper") Disc("rcDisc")"
    say hLine
  end
  /* say "---> Hit <ENTER> to return"   */
  /*  pull .                            */
  /* Call SysDumpVariables "MyVars.txt" */
Return
