/* ------------------------------------------------------------ */
/* MqLineUp V001.001 --->                       09.03.2009, und */
/* WebSphere MQ PUT application                                 */
/* ------------------------------------------------------------ */
/* Requires: RXMQT.dll (Rexx-MQ-Routine-Client)                 */
/*           RXMQN.dll (Rexx-MQ-Routine-Server)                 */
/* WMQ Support Pac MA95. Version 1.0 (C)                        */
/* Copyright IBM Corporation. 1997, 2008.                       */
/* ------------------------------------------------------------ */
/* - Processes files in input directory                         */
/* - Generate one message per line (end with CRLF               */
/* ------------------------------------------------------------ */
parse arg parms

/* ------------------------------------------------------------ */
call FileSpecs        /* input and output file specification    */
call InitFields       /* init fields                            */
call InitFunctions    /* init MQ                                */
call ConnectToMq      /* connect to MQ and open queue           */
if rcConn = 0 & rcOpen = 0
then call ProcessFiles/* process .txt files in directory        */
call EndProcedure     /* close files an terminate MQ            */
exit                  /* End                                    */
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
FileSpecs:
/* ------------------------------------------------------------ */
  file_spec   = "*.*"
  inp_path    = "H:\_UK\MQ-Tools\Input\"
  log_path    = "H:\_UK\MQ-Tools\Log\"
  archiv_path = "H:\_UK\MQ-Tools\Archiv\"
  log_file    = "MQUpload.log"
  logFile = .stream~new(log_path || log_file)
  do until log_open = "READY:"
    log_open = logFile~open
    if log_open <> "READY:"
    then do
      say "RC-LogOpen =" log_open
      say "... waiting 5 sec..."
      Call SysSleep 5
    end
  end
  call sysfiletree inp_path || file_spec, 'file', FO
  if file.0 = 0
  then do
    say "===> no Files found for MQ-Upload:"
    pull .
    exit -9999  /* leave if directory is empty */
  end
  say "===> Files found for MQ-Upload:" file.0
return

/* ------------------------------------------------------------ */
InitFields:
/* ------------------------------------------------------------ */
  if parms~length = 0
  then do
    say "===================================================================>"
    say "===> Enter QueueName for MQPUT and environment (FFM/ANL)"
    pull parms
  end
  if parms~length = 0 then exit -0815
  parms = parms~translate
  parse var parms queue_name mq_env
  if mq_env = "" then mq_env = "FFM"
  if mq_env = "FFM"
  then do
    queue_mgr = "MQFFM01"    
    "SET MQSERVER=MQFFM01.CH/TCP/ibfftocw22224.de.AD.drkw.net(1414)" 
  end
  if mq_env = "ANL"
  then do
    queue_mgr = "MQANL51"
    "SET MQCCSID=850"
    "SET MQSERVER=MQANL51.CH/TCP/anl5.vme.vse.dresdner.net"
  end
  if mq_env = "MQWIE2"
  then do
    queue_mgr = "MQWIE2"    
    "SET MQSERVER=MQWIE2T.CH/TCP/ff0p5300apx.de.dresdnerkb.com(1415)"
  end
  if mq_env = "MQWIE8"
  then do
    queue_mgr = "MQWIE8"    
    "SET MQSERVER=MQWIE8T.CH/TCP/ff0p5300apx.de.dresdnerkb.com(1414)"
  end
  say "===================================================================>"
  say "===> QueueManager is" queue_mgr
  LogMsg.1 = "Application: RxMQPUT , Target: QM =" queue_mgr", Queue =" queue_name
  LogMsg.2 = "-"~copies(80)
  LogMsg.0 = 1
  say "===>" LogMsg.1
  call WriteToLog
  fileCount = 0 ; byteCount = 0
return

/* ------------------------------------------------------------ */
InitFunctions:
/* ------------------------------------------------------------ */
  call rxfuncadd 'sysloadfuncs', 'rexxutil', 'sysloadfuncs'
  call sysloadfuncs
  rcc = RxFuncadd('RXMQINIT','RXMQT','RXMQINIT')
  parse var rcc rcFunc .
  if rcFunc <> 0 & rcFunc <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Load MQ Functions"
    call WriteToLog
    exit -3333
  end
  rcc = RXMQINIT()
  rcc = RXMQCONS()
  parse var rcc rcInit .
  if rcInit <> 0 & rcInit <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Init MQ Functions"
    call WriteToLog
    exit -4444
  end
return

/* ------------------------------------------------------------ */
ConnectToMq:
/* ------------------------------------------------------------ */
  rcc = RXMQCONN(queue_mgr)
  parse var rcc rcConn .
  if rcConn = 0
  then do
    iod.ot = MQOT_Q
    rcc = RXMQOPEN(queue_name, mqoo_output, 'ObjHndl', 'ood.')
    parse var rcc rcOpen .
    if rcOpen <> 0
    then do
      LogMsg.0 = 2
      LogMsg.1 = "MQOPEN failed with" rcc
      call WriteToLog
      exit -4444
    end
  end
  else do
    LogMsg.0 = 2
    LogMsg.1 = "MQCONN failed with" rcc
    call WriteToLog
    exit -4444
  end
return

/* ------------------------------------------------------------ */
ProcessFiles:
/* ------------------------------------------------------------ */
  do i = 1 to file.0
    lineCount = 0 ; byteCount = 0
    in_file      = file.i
    in_file_name = in_file~substr(in_file~lastpos("\") + 1)
    in_file_path = in_file~substr(1,in_file~lastpos("\"))
    if in_file_name~lastpos(".") > 0
    then in_file_noex = in_file_name~substr(1,in_file_name~lastpos(".") - 1)
    else in_file_noex = in_file_name
    in_file_err  = in_file_path || in_file_noex || ".err"
    err_file     = in_file_path || in_file_err
    mq_inp = .stream~new(in_file)
    rc = mq_inp~open('read')
    if rc <> 'READY:'
    then do
      LogMsg.0 = 1
      LogMsg.1 = "Error open Input file" in_file_name
      call WriteToLog
      iterate
    end
    do while mq_inp~lines > 0
      g.1 = mq_inp~linein
      g.0 = g.1~length
      if mq_env = "ANL"
      then do
        call 'RxTrans' g.1, a, e
        g.1 = result
      end
      ipmd.MSG = MQMT_DATAGRAM
      ipmd.form = 'MQSTR'
      ipmd.PER = MQPER_PERSISTENT
      ipmo.opt = MQPMO_LOGICAL_ORDER + MQPMO_SYNCPOINT + MQPMO_NO_CONTEXT + MQPMO_FAIL_IF_QUIESCING
      ipmo.opt = MQPMO_LOGICAL_ORDER + MQPMO_SYNCPOINT
      rcc = RXMQPUT( ObjHndl,'g.','ipmd.','opmd.','ipmo.','opmo.')
      parse var rcc rcPut .
      if rcPut = 0
      then do
        rcc = RXMQCMIT()
        lineCount = lineCount + 1
        byteCount = byteCount + g.0
      end
      else do
        LogMsg.0 = 2
        LogMsg.1 = "MQPUT ["in_file_name"] failed with" rcc "[Line:" || lineCount || ":"byteCount":]"
        call WriteToLog
        exit -4444
      end
    end
    LogMsg.0 = 1
    LogMsg.1 = "MQPUT ["in_file_name"] completed with rc" rcPut". Lines:" lineCount "Bytes:" byteCount
    call WriteToLog
    say LogMsg.1
    mq_inp~close
    if rcPut = 0
    then rcMov = SysFileMove(in_file, archiv_path || in_file_name) /* "MOVE" in_file archiv_path */
    else do
      rcRen = SysFileCopy(in_file, in_file_err)
      rcRen = SysFileDelete(in_file)
    end
    if rcMov <> 0
    then do
      LogMsg.0 = 2
      LogMsg.1 = "MOVE failed with RC" rcMov "["in_file_name"]"
      call WriteToLog
      exit -5555
    end
  end
return

/* --------------------------------------------------------- */
/* Write to logfile                                          */
WriteToLog:
/* --------------------------------------------------------- */
  TimeStamp = "["date()~right(11) time()"] --->"
  if log_open <> "READY:" then exit -7777
  if LogMsg.0 > 0
  then do LogCount = 1 to LogMsg.0
    LogFile~lineout(TimeStamp LogMsg.LogCount)
  end
return WriteToLog

/* ------------------------------------------------------------ */
EndProcedure:                 /* close files an terminate MQ    */
/* ------------------------------------------------------------ */
  LogMsg.0 = 1
  LogMsg.1 = LogMsg.2
  call WriteToLog
  LogFile~close
  rcc = RXMQCLOS(ObjHndl, mqco_none)
  rcc = RXMQDISC()
  say "===================================================================>"
return
